searchData={"content_type":"text/markdown","items":[{"doc":"`TwoFactorInACan` provides easy to use functions for implementing two factor\nauthentication in a phoenix application.","ref":"TwoFactorInACan.html","title":"TwoFactorInACan","type":"module"},{"doc":"Provides functions for working with the HMAC-based One Time Password\nalgorithm as defined in RFC 4226.\n\nFor details on RFC 4226, see https://tools.ietf.org/rfc/rfc4226.txt.","ref":"TwoFactorInACan.Hotp.html","title":"TwoFactorInACan.Hotp","type":"module"},{"doc":"Generates a token from a shared secret and a counter which can be\nsynchronized.\n\nThis token can be used by one party to verify whether another party has the\nsame secret.\n\nOptions:\n- `:secret_format` - the format that the secret is passed in as. Options\n  include:\n  - `:binary` (default)\n  - `:base32`\n  - `:base64`\n- `:token_length` (Default: 6) - the length of the generated token. A longer\n  token is harder to guess and thus more secure. A longer token can also be\n  more difficult for users to accurately transmit. Although everything in\n  `TwoFactorInACan` supports variable token length, you should be sure that\n  other apps and programs used support the token length set here.","ref":"TwoFactorInACan.Hotp.html#generate_token/3","title":"TwoFactorInACan.Hotp.generate_token/3","type":"function"},{"doc":"```elixir\niex> secret = TwoFactorInACan.Secrets.generate_totp_secret()\niex> TwoFactorInACan.Hotp.generate_token(secret, 0)\n\"866564\"\n\niex> TwoFactorInACan.Hotp.generate_token(secret, 1)\n\"532769\"\n\niex> TwoFactorInACan.Hotp.generate_token(secret, 0, token_length: 10)\n\"1807866564\"\n```","ref":"TwoFactorInACan.Hotp.html#generate_token/3-examples","title":"Examples - TwoFactorInACan.Hotp.generate_token/3","type":"function"},{"doc":"Verifies that the provided HOTP token was generated using the provided secret\nand count.\n\nThis function uses the secret and count to generate a token. It then compares\nthat generated token to the passed in token. If they match, then this\nfunction returns true. If they do not match, then this function returns\nfalse.\n\nThis function allows a number of options:\n- `:secret_format` - the format that the secret is passed in as. Options\n  include:\n  - `:binary` (default)\n  - `:base32`\n  - `:base64`\n- `:token_length` (Default: 6) - the length of the generated token. A longer\n  token is harder to guess and thus more secure. A longer token can also be\n  more difficult for users to accurately transmit. Although everything in\n  `TwoFactorInACan` supports variable token length, you should be sure that\n  other apps and programs used support the token length set here.","ref":"TwoFactorInACan.Hotp.html#same_secret?/4","title":"TwoFactorInACan.Hotp.same_secret?/4","type":"function"},{"doc":"Functions for generating cryptographic secrets.","ref":"TwoFactorInACan.Secrets.html","title":"TwoFactorInACan.Secrets","type":"module"},{"doc":"Generates a secret suitable for use in time based one time password (TOTP)\ntwo factor authentication.\n\nGenerates a 160-bit key which is the size recommended by RFC4226\n(https://www.ietf.org/rfc/rfc4226.txt).\n\nThe following options are supported:\n- `:format` (Default: `:binary`) - The format of the generated secret.\n  Options include:\n  - `:binary` (default)\n  - `:base32`\n  - `:base64`\n- `:bytes` (Default: `20`) - The size in bytes of the generated secret. 20\n  bytes (160 bits) is the size specified by RFC 4226. It is also the size\n  supported by most authenticator applications.\n\n# Examples\n\n    iex> TwoFactorInACan.Secrets.generate_totp_secret()\n    <<195, 110, 253, 36, 185, 138, 174, 16, 54, 176, 135, 67, 97, 11, 159, 63, 75, 80, 65, 6>>\n\n    iex> TwoFactorInACan.Secrets.generate_totp_secret(format: :base32)\n    \"F2EJJEYSJA67QHI6DEAI2I6AGCEG7G5E\"\n\n    iex> TwoFactorInACan.Secrets.generate_totp_secret(format: :base64)\n    \"xKXOSYcRVlHfnazLMlRinpb252U=\"","ref":"TwoFactorInACan.Secrets.html#generate_totp_secret/1","title":"TwoFactorInACan.Secrets.generate_totp_secret/1","type":"function"},{"doc":"","ref":"TwoFactorInACan.Secrets.html#t:secret_generation_opts/0","title":"TwoFactorInACan.Secrets.secret_generation_opts/0","type":"type"},{"doc":"Provides functions for working with time based one time password (TOTP) style\ntwo factor authentication (2FA) as defined in RFC 4226.","ref":"TwoFactorInACan.Totp.html","title":"TwoFactorInACan.Totp","type":"module"},{"doc":"For details on RFC 4226, see https://tools.ietf.org/rfc/rfc4226.txt.\n\nTOTP two factor authentication uses the HMAC-based one time password (HOTP)\nalgorithm with the current time passed in as the count to verify that the end\nuser and the consuming application have the same secret.\n\nIt does this by generating a token from the current time and secret and\ncomparing the generated secret to the one the end user has supplied. If they\nmatch, then it is extremely unlikely that the end user does not have a\ndifferent secret.","ref":"TwoFactorInACan.Totp.html#module-summary","title":"Summary - TwoFactorInACan.Totp","type":"module"},{"doc":"When implementing 2FA using TOTP it is very important to also lock out any\nusers which do not send a matching token some small number of consecutive\ntimes. By default tokens are 6 characters long which means that there is a\n1/1,000,000 chance of guessing correctly. This is relatively trivial to brute\nforce without some lock out mechanism.\n\nCare should also be token when adjusting the token length. Longer tokens are\nmore secure, and shorter tokens are less secure (as they are easier to\nguess).\n\nIf there is drift between clocks of users and the machine verifying tokens\nthen the `acceptable_past_tokens` and `acceptable_future_tokens` options can\nbe used to specify a window of acceptable tokens. This can also be useful to\nallow users to still submit tokens which were just seconds ago valid as a\nuser experience enhancement.\n\nIt should be noted that increasing the window of acceptable tokens also makes\nit easier to randomly guess a token. Care should be taken to way the pros and\ncons of this. As an example, by allowing one past token and one future token\nto be valid, there are now (usually) 3 tokens that an attacker could guess.\nThis increases the chance of successfully guessing tokens to 1/333,333.","ref":"TwoFactorInACan.Totp.html#module-security-concerns","title":"Security Concerns - TwoFactorInACan.Totp","type":"module"},{"doc":"","ref":"TwoFactorInACan.Totp.html#module-example-usage","title":"Example Usage - TwoFactorInACan.Totp","type":"module"},{"doc":"When a user first sets up TOTP 2FA, the server must generate a secret that\nwill be shared between the server and the user. The `TwoFactorInACan.Secrets`\nmodule can be used for this.\n\n```elixir\nsecret = TwoFactorInACan.Secrets.generate_totp_secret()\n<<109, 159, 180, 42, 128, 80, 183, 56, 163, 232, 151, 242, 233, 37, 167, 178,\n  253, 23, 18, 159>>\n```\n\nEvery user should have a different secret. No secrets should be shared.","ref":"TwoFactorInACan.Totp.html#module-secret-generation","title":"Secret Generation - TwoFactorInACan.Totp","type":"module"},{"doc":"This secret must be securely transmitted to the user. It is extremely\nimportant that the method of transfer is secure. If an attacker can intercept\nthe secret in transit, then they will be able to entirely bypass this form of\n2FA. The attacker then only needs to acquire the user's password and this 2FA\nmethod is rendered useless.\n\nUsing HTTPS is a must for transferring this secret. Do not transfer anything\nover HTTP!\n\nA common method of transferring the secret to an end user is by generating a\nQR code containing the secret. Smart phone applications such as Google\nAuthenticator, Duo, and Authy can then scan these codes, extract the secret,\nand securely store the secret.\n\nFor these applications to understand that a secret is being transmitted, the\nfollowing url should be encoded to a QR Code:\n\n```elixir\n\"otpauth://totp/MyDescription?secret=MySecret&issuer=MyAppName\"\n|> MyQRModule.encode # Note: Not a real module!\n```\n\nIn the above URL, MySecret should be the secret encoded in base32, MyAppName\nand MyDescription can be anything, and assist the user in figuring out which\ntoken to use on your site if they have many.","ref":"TwoFactorInACan.Totp.html#module-transmitting-the-secret-to-the-user","title":"Transmitting the Secret to the User - TwoFactorInACan.Totp","type":"module"},{"doc":"At this point the end user should be asked to supply the current token as\ngenerated by their authenticator application.\n\nUpon receiving the token from the user, it should be verified against the\nsecret generated earlier.\n\n```elixir\nTwoFactorInACan.Totp.same_secret?(generated_secret, user_supplied_token)\ntrue\n```\n\nIf the above function returns `true` then the the user has properly setup\n2FA.\n\nIf it returns `false`, then the incorrect token was sent. The user will need\nto send another correct token before their account should be configured to\nrequire TOTP 2FA at login.","ref":"TwoFactorInACan.Totp.html#module-verify-user-received-secret","title":"Verify User Received Secret - TwoFactorInACan.Totp","type":"module"},{"doc":"The server will need to store the generated secret in some manner associated\nwith the user so that it can be used to verify the user's identity at login.\n\nCare should be taken to ensure this is stored securely. If storing in a\ndatabase, the database should be encrypted. Even if the database is\nencrypted, the field should be encrypted. You can even go the extra mile and\nuse a different encryption key per user stored with the key stored outside of\nthe database.\n\nIf the secret is leaked or compromised by an attacker, then the attacker will\nbe able to bypass this method of 2FA. In this event, the user should be\nprompted to re-setup TOTP 2FA. Ideally the user's account should be locked\ndown until they can prove their identity through other means.","ref":"TwoFactorInACan.Totp.html#module-store-the-secret","title":"Store the secret - TwoFactorInACan.Totp","type":"module"},{"doc":"When the user logs in, they should also supply the current TOTP token. The\nserver should verify that the supplied TOTP token is generated using the same\nsecret that is stored associated with that user.\n\n```elixir\nTwoFactorInACan.Totp.same_secret?(stored_user_secret, user_supplied_token)\ntrue\n```\n\nIf this returns true, then the user has sufficiently proven their identity\nand authentication should succeed (assuming they also supplied the correct\npassword!)\n\nIf this returns false, then login should fail.\n\nIt can be a good practice to provide the user with a message that does not\nreveal whether the username didn't exist, the password was wrong, or 2FA\nfailed. This makes an attackers job more difficult. There is some debate on\nwhether this is an effective security measure.","ref":"TwoFactorInACan.Totp.html#module-verify-token-at-login","title":"Verify Token at Login - TwoFactorInACan.Totp","type":"module"},{"doc":"Outputs the current TOTP token for the given secret.\n\nExpects a 160-bit binary key by default. To use secrets that are encoded the\n`:secret_format` option can be supplied. It expects one of the following\nvalues:\n- `:binary` (default)\n- `:base32`\n- `:base64`\n\nThe following options are supported:\n- `:secret_format` - the format that the secret is passed in as. Options\n  include:\n  - `:binary` (default)\n  - `:base32`\n  - `:base64`\n- `:token_length` (Default: 6) - the length of the generated token. A longer\n  token is harder to guess and thus more secure. A longer token can also be\n  more difficult for users to accurately transmit. Although everything in\n  `TwoFactorInACan` supports variable token length, you should be sure that\n  other apps and programs used support the token length set here.\n- `:offset_seconds` (Default: 0) - The number of seconds to offset the\n  current timestamp by. If the current timestamp was 600 and the offset was\n  60, then the timestamp of 660 would be used to calculate the time interval.\n  This can be useful to account for drift or to purposefully allow the last\n  token to be valid as well in functions that use this function.\n- `:interval_seconds` (Default: 30) - The number of seconds that must pass\n  before a new time_interval (and thus TOTP token) is returned by this\n  function. This should probably never be anything but the default (30\n  seconds) during actual use, as nearly all apps that generate TOTP tokens\n  assume a 30 second interval.\n- `:injected_timestamp` (default: current time) - The unix timestamp to use\n  when calculating the time interval. This should only be used during testing\n  to ensure the same token or interval is always returned. When this option\n  is not supplied, the current timestamp is used.\n\n# Examples\n\n```elixir\niex> secret = TwoFactorInACan.Secrets.generate_totp_secret()\niex> TwoFactorInACan.Totp.current_token_value(secret)\n\"858632\"\n\niex> secret = TwoFactorInACan.Secrets.generate_totp_secret(format: :base32)\niex> TwoFactorInACan.Totp.current_token_value(secret, secret_format: :base32)\n\"743622\"\n\niex> secret = TwoFactorInACan.Secrets.generate_totp_secret(format: :base64)\niex> TwoFactorInACan.Totp.current_token_value(secret, secret_format: :base64)\n\"384012\"\n```","ref":"TwoFactorInACan.Totp.html#current_token_value/2","title":"TwoFactorInACan.Totp.current_token_value/2","type":"function"},{"doc":"Verifies that the provided TOTP token was generated using the provided\nsecret.\n\nThis function uses the secret to generate a token. It then compares the\ngenerated token to the supplied token. If they match, then it can be\nprobabilistically inferred that the entity that supplied the token also knew\nthe secret.\n\nThis function allows a number of options:\n- `:acceptable_past_tokens` (Default: `0`) - The number of past tokens which\n  should result in this function returning true. Setting this to `1` can be a\n  friendly way to allow users to still verify if they have taken too long to\n  submit their token. It should be noted that this does result in two tokens\n  being valid instead of one, which makes a valid token easeier to guess.\n  This value should not be set too high or security is greatly compromised.\n- `:acceptable_future_tokens` (Default: `0`) - The number of future tokens\n  which should result in this function returning true. It should be noted\n  that setting this to a nonzero value will allow more tokens to be valid and\n  thus make a valid token easier to guess. This value should not be set too\n  high or security is greatly compromised.\n- `:token_length` (Default: 6) - the length of the generated token. A longer\n  token is harder to guess and thus more secure. A longer token can also be\n  more difficult for users to accurately transmit. Although everything in\n  `TwoFactorInACan` supports variable token length, you should be sure that\n  other apps and programs used support the token length set here.\n- `:offset_seconds` (Default: `0`) - The number of seconds to offset the\n  current timestamp by. If the current timestamp was 600 and the offset was\n  60, then the timestamp of 660 would be used to calculate the time interval.\n  This can be useful to account for drift or difference in clock times\n  between two entities.\n- `:interval_seconds` (Default: `30`) - The number of seconds that must pass\n  before a new time_interval (and thus TOTP token) is returned by this\n  function. This should probably never be anything but the default (30\n  seconds) during actual use, as nearly all apps that generate TOTP tokens\n  assume a 30 second interval.\n- `:injected_timestamp` (default: current time) - The unix timestamp to use\n  when calculating the time interval. This should only be used during testing\n  to ensure the same token or interval is always returned. When this option\n  is not supplied, the current timestamp is used.\n- `:secret_format` (default: `:binary`) - The format of the passed in secret.\n  Can be one of `:binary`, `:base32`, or `:base64`.","ref":"TwoFactorInACan.Totp.html#same_secret?/3","title":"TwoFactorInACan.Totp.same_secret?/3","type":"function"},{"doc":"```elixir\niex> secret = TwoFactorInACan.Secrets.generate_totp_secret()\niex> current_token = TwoFactorInACan.Totp.current_token_value(secret)\niex> TwoFactorInACan.Totp.same_secret?(secret, current_token)\ntrue\n```","ref":"TwoFactorInACan.Totp.html#same_secret?/3-examples","title":"Examples - TwoFactorInACan.Totp.same_secret?/3","type":"function"},{"doc":"Calculates the current time interval as an integer used in the HOTP algorithm\nas the count to calculate the current token value.\n\nThe following options are supported:\n- `:offset_seconds` (Default: 0) - The number of seconds to offset the\n  current timestamp by. If the current timestamp was 600 and the offset was\n  60, then the timestamp of 660 would be used to calculate the time interval.\n  This can be useful to account for drift or to purposefully allow the last\n  token to be valid as well in functions that use this function.\n- `:interval_seconds` (Default: 30) - The number of seconds that must pass\n  before a new time_interval (and thus TOTP token) is returned by this\n  function. This should probably never be anything but the default (30\n  seconds) during actual use, as nearly all apps that generate TOTP tokens\n  assume a 30 second interval.\n- `:injected_timestamp` (default: current time) - The unix timestamp to use\n  when calculating the time interval. This should only be used during testing\n  to ensure the same token or interval is always returned. When this option\n  is not supplied, the current timestamp is used.","ref":"TwoFactorInACan.Totp.html#time_interval/1","title":"TwoFactorInACan.Totp.time_interval/1","type":"function"},{"doc":"```elixir\niex> TwoFactorInACan.Totp.time_interval()\n51802243\n\niex> TwoFactorInACan.Totp.time_interval(offset_seconds: 30)\n51802244\n\niex> TwoFactorInACan.Totp.time_interval(interval_seconds: 60)\n25901122\n\niex> TwoFactorInACan.Totp.time_interval(injected_timestamp: 1554067403)\n51802246\n\niex> TwoFactorInACan.Totp.time_interval(injected_timestamp: 60, interval_seconds: 10)\n6\n```","ref":"TwoFactorInACan.Totp.html#time_interval/1-examples","title":"Examples - TwoFactorInACan.Totp.time_interval/1","type":"function"},{"doc":"# Getting Started","ref":"getting-started.html","title":"Getting Started","type":"extras"},{"doc":"`TwoFactorInACan` is a library for adding two factor authentication (2FA) to\nyour application. Its goal is to be the most developer friendly way to securely\nsetup two factor authentication in the Elixir/Erlang ecosystem. That means that\nnot only is it thoroughly documented and easy to add to your project, but it's\nalso the easiest to understand should you need to take a look behind the\ncurtain.","ref":"getting-started.html#summary","title":"Summary - Getting Started","type":"extras"},{"doc":"This library is not currently available on hex.\n\nOnce it is:\n\nThe package can be installed by adding `two_factor_in_a_can` to your list of\ndependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:two_factor_in_a_can, \"~> 0.2.0\"}\n  ]\nend\n```","ref":"getting-started.html#installation","title":"Installation - Getting Started","type":"extras"},{"doc":"","ref":"getting-started.html#usage","title":"Usage - Getting Started","type":"extras"},{"doc":"See `TwoFactorInACan.Totp` for usage details.","ref":"getting-started.html#time-based-one-time-password-totp","title":"Time-Based One Time Password (TOTP) - Getting Started","type":"extras"},{"doc":"See `TwoFactorInACan.Hotp` for usage details.","ref":"getting-started.html#hmac-based-one-time-password-hotp","title":"HMAC-Based One Time Password (HOTP) - Getting Started","type":"extras"},{"doc":"","ref":"roadmap.html","title":"Roadmap","type":"extras"},{"doc":"This project is still very, very, very much under construction.\n\nThat said, here's what's planned:\n- [ ] HMAC based One Time Password 2FA (HOTP)\n  - [x] Secret Generation\n  - [x] `Hotp.generate_token/3` function\n  - [x] `Hotp.same_secret?/3` function\n  - [ ] Write documentation\n- [x] Time-based One Time Password 2FA (TOTP)\n  - [x] Secret Generation\n  - [x] `Totp.current_token_value/2` function\n  - [x] `Totp.same_secret?/3` function\n  - [x] Write documentation\n- [ ] Easy TOTP QR Code generation\n  - [ ] with nonce\n  - [ ] with sha256\n- [ ] Universal Second Factor (U2P)\n- [ ] Add examples to `README.md`\n- [ ] Tutorial for adding to Phoenix","ref":"roadmap.html#roadmap","title":"Roadmap - Roadmap","type":"extras"},{"doc":"2FA via text to someone's phone (SMS) is better than no 2FA at all, but there\nare security users to be aware of. The main one is that it is not impossible to\nintercept the communication of the token when sent to the user's phone number.\nUltimately, there are no cryptographic guarantees that the device used to\nauthenticate is the same device that was originally used to setup 2FA.\n\n- [ ] SMS based 2FA\n  - [ ] Thoroughly warn about not using\n    - [ ] But note why its still better than no 2FA at all\n  - [ ] Build OTP process tree for keeping state of recently sent SMS\n  - [ ] `Sms.generate_and_send_token/2` function\n  - [ ] `Sms.verify_token/3` function\n  - [ ] Document to pieces","ref":"roadmap.html#maybe-but-probably-not","title":"Maybe, but probably not - Roadmap","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}